__all__ = ('Client',)

from typing import Optional
from .aiorequests import *
from .guild import Guild

URL = r'https://api.sgroup.qq.com'


class Client:
    def __init__(self, app_id: str, token: str):
        self.token = token
        self.app_id = app_id
        self.auth = {'Authorization': f'Bot {self.app_id}.{self.token}'}

    async def get_guild(self, guild_id: str) -> Optional[Guild]:
        r = await get(URL + f'/guilds/{guild_id}', headers=self.auth)
        return Guild(await r.json())

    async def get_all_guid(self) -> list[Guild]:
        r = await get(URL + '/users/@me/guilds', headers=self.auth)
        return [Guild(i) for i in await r.json()]



from typing import (
    Any,
    ClassVar,
    Dict,
    List,
    NamedTuple,
    Sequence,
    Set,
    Literal,
    Optional,
    TYPE_CHECKING,
    Tuple,
    Union,
    overload,
)


from .types.guild import Guild as GuildPayload
from .types.channel import VoiceChannel, TextChannel, CategoryChannel

VocalGuildChannel = Union[VoiceChannel]
GuildChannel = Union[VoiceChannel, TextChannel, CategoryChannel]


class Guild:
    __slots__ = (
        'id',
        'name',
        'icon',
        'owner_id',
        'owner',
        'member_count',
        'max_members',
        'description',
        'joined_at',
        '_channels'
    )

    def __init__(self, data: GuildPayload):
        self._channels: Dict[int, GuildChannel] = {}
        self._from_data(data)

    def _from_data(self, guild: GuildPayload) -> None:
        # according to Stan, this is always available even if the guild is unavailable
        # I don't have this guarantee when someone updates the guild.
        self.id = guild.get('id')
        self.name = guild.get('name')
        self.icon = guild.get('icon')
        self.owner_id = guild.get('owner_id')
        self.owner = guild.get('owner')
        self.member_count = guild.get('member_count')
        self.max_members = guild.get('max_members')
        self.description = guild.get('description')
        self.joined_at = guild.get('joined_at')
